/*	$ JBOSN : touch.c, v 0.1 2003/09/01 $	*/

/*
 * Copyright (C) 2003	Jung Byong-Oh
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are not permitted completely.
 *
 */

/*
 * Module Name: touch.c
 *
 * Purpose: touch pad interface for graphic system of j-bosn
 */

#include <types.h>
#include <thread.h>
#include <message.h>
#include <interrupt.h>
#include <device.h>
#include <error.h>

#include <window.h>

#include <hal_panic.h>

#include "touch.h"

/*
typedef struct _INPUTEVENT_MOUSE       INPUTEVENT_MOUSE;
typedef struct _INPUTEVENT_MOSUE      *PINPUTEVENT_MOUSE;
struct _INPUTEVENT_MOUSE
{
    UINT32      State;
        #define     MOUSE_STATE_LEFT        (1<<0)
        #define     MOUSE_STATE_RIGHT       (1<<1)
        #define     MOUSE_STATE_MIDDLE      (1<<2)
        #define     MOUSE_STATE_MOVE        (1<<3)
        #define     MOUSE_STATE_UP          (1<<4)
        #define     MOUSE_STATE_DOWN        (1<<5)
        
        #define     MOUSE_STATE_LEFTDOWN        (MOUSE_STATE_LEFT|MOUSE_STATE_DOWN)
        #define     MOUSE_STATE_LEFTUP          (MOUSE_STATE_LEFT|MOUSE_STATE_UP)
        #define     MOUSE_STATE_RIGHTDOWN       (MOUSE_STATE_RIGHT|MOUSE_STATE_DOWN)
        #define     MOUSE_STATE_RIGHTUP         (MOUSE_STATE_RIGHT|MOUSE_STATE_UP)
        #define     MOUSE_STATE_MIDLLEDOWN      (MOUSE_STATE_MIDDLE|MOUSE_STATE_DOWN)
        #define     MOUSE_STATE_MIDDLEUP        (MOUSE_STATE_MIDDLE|MOUSE_STATE_UP)
    UINT32      X;
    UINT32      Y;
};
typedef struct _INPUTEVENT_KEY         INPUTEVENT_KEY;
typedef struct _INPUTEVENT_KEY        *PINPUTEVENT_KEY;
        struct _INPUTEVENT_KEY
        {
            UINT32      State;
                #define     KEY_STATE_PUSHED        (0x01)
                #define     KEY_STATE_RELEASED      (0x02)
            UINT32      vKey;
            UINT32      vScan;
        };


typedef struct _INPUTEVENT
{
    UINT32      Type;
        #define INPUTEVENT_TYPE_MOUSE       (0x1)
        #define INPUTEVENT_TYPE_KEY         (0x2)

    union 
    {
        INPUTEVENT_MOUSE    Mouse;
        INPUTEVENT_KEY      Key;
    }u;
}
INPUTEVENT, *PINPUTEVENT;
*/



//=======================================================

STATIC INPUTEVENT   TouchInputEvent;
STATIC UINT32       nSysIntr;

STATIC UINT32   Xmouse;
STATIC UINT32   Ymouse;
STATIC UINT32   Smouse;


#ifdef DISPLAY_ROT

    INT32       Xscr[MAX_NCALIBRATION] ={160,  64,  64, 256, 256};
    INT32       Yscr[MAX_NCALIBRATION] ={120,  48, 192, 192,  48};
    
    INT32       Xtch[MAX_NCALIBRATION] ={2056,  936, 940, 3176, 3154};
    INT32       Ytch[MAX_NCALIBRATION] ={2070, 3141, 992,  956, 3131};

#else

    INT32       Xscr[MAX_NCALIBRATION] ={120,  48,  48, 192, 192};
    INT32       Yscr[MAX_NCALIBRATION] ={160,  64, 256, 256,  64};
    
    INT32       Xtch[MAX_NCALIBRATION] ={2070,  992,  956, 3131, 3141};
    INT32       Ytch[MAX_NCALIBRATION] ={2056,  940, 3176, 3154,  936};

#endif

// INT32       Xtch[MAX_NCALIBRATION] ={520, 858, 860, 179, 176};
// INT32       Ytch[MAX_NCALIBRATION] ={512, 782, 233, 229, 779};
// //
// ////INT32       Xtch[MAX_NCALIBRATION] ={520, 179, 176, 858, 860};
// ////INT32       Ytch[MAX_NCALIBRATION] ={512, 229, 809, 812, 233};

//UINT32      TchInterruptState = 0;
//BOOL        bCalibrated = FALSE;
//
UINT32      Channel;
STATIC BOOL th_TouchDevice( UINT32 Arg );

//#define ABS(a)              ((a)>0 ? (a): -(a))
//#define DELTA_SAMPLE        (40)
//
//

UINT32 TouchDisplayWidth = DISPLAY_WIDTH;
UINT32 TouchDisplayHeight = DISPLAY_HEIGHT;

//=======================================================

#define TOUCHDEV_STACK_SIZE     (4096)
BYTE    TouchDev_Stack[TOUCHDEV_STACK_SIZE];

DEVICECONFIG            Touch_DeviceConfig ={"TCH", 0, -1 };

THREADCONFIG TouchDev_Thread =
    {(UINT32)&Touch_DeviceConfig, th_TouchDevice, TouchDev_Stack, TOUCHDEV_STACK_SIZE, THREAD_PRIORITY_DRIVER+10};

//=======================================================




/*
  Name: th_TouchDevice()
  Copyright: 
  Author: jung byoung oh
  Date: 15-12-03 11:37
  Description: block device thread
  Command:
    MESSAGE_TYPE_INTERRUPT
    MESSAGE_TYPE_DEVICE_READ
    MESSAGE_TYPE_DEVICE_WRITE
*/
PUBLIC BOOL
th_TouchDevice( UINT32 Arg )
{
    JB_RECVMESSAGE      Message;
    HANDLE              hPort;
    UINT32              AccessMode = PORTTYPE_MESSAGE|PORTTYPE_INTERRUPT|PORTTYPE_SIGNAL;

    hPort = PortGet( );    
    PortSetType(hPort, AccessMode);    // Only Receive the request

    DEBUG2(1,"th_TouchDevice %X\n",Arg);

    Touch_DeviceConfig.Name[3] = (UINT32)Touch_DeviceConfig.Minor + '1';
    Touch_DeviceConfig.Channel = Channel = ChannelGet( hPort );
    DEBUG2(1,"th_TouchDevice p_DeviceConfig->Channel = %X\n",Touch_DeviceConfig.Channel);
    DeviceRegister( &Touch_DeviceConfig );
    DEBUG2(1,"th_TouchDevice p_DeviceConfig->Channel = %X\n",Touch_DeviceConfig.Channel);

    nSysIntr = HAL_TouchInit();

    InterruptRegister(nSysIntr, hPort);
    DEBUG2(1,"th_TouchDevice hPort = %X\n",hPort);



    TouchCalibration( Xscr, Yscr, Xtch, Ytch);

//{
//    INT32 Xscr, Yscr;
//
//    TouchCalibrate( 858, 851, &Xscr, &Yscr );
//    
//    DEBUG2(1," TouchCalibrate X = %D\n", Xscr );
//    DEBUG2(1," TouchCalibrate Y = %D\n", Yscr );
//}
    
    InterruptEnable(nSysIntr);

    while(1)
    {
        MessageReceive( hPort, &Message);

        switch(Message.Type)
        {

        case MESSAGE_TYPE_INTERRUPT:
        {

            UINT32                      fSample = 0;
            INT32                       RawX, CalX;
            INT32                       RawY, CalY;
        
            STATIC UINT32               bPrevPenDown = FALSE;

            //DEBUG2(1," MESSAGE_TYPE_INTERRUPT  = %D\n",  MESSAGE_TYPE_INTERRUPT);
        
            // Give the pdd the down state of the previous sample
            if (bPrevPenDown)
                fSample |= TOUCH_SAMPLE_PREVIOUSDOWN;
            else
                fSample &= ~TOUCH_SAMPLE_PREVIOUSDOWN;
        
            HAL_TouchGetPoint( &fSample, &RawX, &RawY );    // Get the point info
//DEBUG2(1," fSample & TOUCH_SAMPLE_VALID  fSample = %x\n",  fSample);
        
            if (fSample & TOUCH_SAMPLE_IGNORE)
                continue;

            if (fSample & TOUCH_SAMPLE_VALID)
            {
                // Set the previous down state for our use, since the pdd may not
                // have preserved it.
                if ( bPrevPenDown )
                    fSample |= TOUCH_SAMPLE_PREVIOUSDOWN;
                else
                    fSample &= ~TOUCH_SAMPLE_PREVIOUSDOWN;

                bPrevPenDown = fSample & TOUCH_SAMPLE_DOWN;
            }
//DEBUG2(1," fSample & TOUCH_SAMPLE_VALID  bPrevPenDown = %x\n",  bPrevPenDown);

//            if ( CalibrationState )
//            {
//                //
//                // At this point we know that calibration is active.
//                //
//                // Typically, the user touches the panel then converges to the
//                // displayed crosshair. When the tip state transitions to
//                // the up state, we forward the last valid point to the callback
//                // function.
//                //
//        
//                //  Skip if not valid.
//                if ( !(fSample & TOUCH_SAMPLE_VALID) )
//                    continue;
//        
//        
//                //  Signal the Power Manager activity event if one has been set up
//                if ( ghevCalibrationActivity != NULL) 
//                    SetEvent(ghevCalibrationActivity);
//        
//                //  Must see down transition.
//                if ( (fSample & (TOUCH_SAMPLE_DOWN|TOUCH_SAMPLE_PREVIOUSDOWN)) == TOUCH_SAMPLE_DOWN )
//                {
//                    CalibrationState = CalibrationDown;
//                    fSetBase = TRUE;
//                    CalibrationSampleCount = 0;
//                    fGotSample = FALSE;
//                }
//        
//                //  Only look at stuff if we saw a down transition.
//                if ( (CalibrationState == CalibrationDown) && !fGotSample )
//                {
//                    if ( fSample & TOUCH_SAMPLE_DOWN )
//                    {
//                        long DeltaX, DeltaY;
//        
//                        CalibrationSampleCount++;
//                        CX = RawX;
//                        CY = RawY;
//                        if ( fSetBase )
//                        {
//                            XBase = CX;
//                            YBase = CY;
//                            BaseTime = GetTickCount();
//                            fSetBase = FALSE;
//                        }
//                        DeltaX = CX - XBase;
//                        DeltaY = CY - YBase;
//                        if ( (GetTickCount() - BaseTime) > CAL_HOLD_STEADY_TIME )
//                        {
//                            fGotSample = TRUE;
//                        }
//                        else
//                        if ( ( ABS(DeltaX) > CAL_DELTA_RESET ) || ( ABS(DeltaY) > CAL_DELTA_RESET ) )
//                        {
//                            RETAILMSG(1, (TEXT("M %ld,%ld  %ld,%ld  %ld,%ld"),
//                            XBase,YBase, CX,CY, DeltaX,DeltaY));
//                            fSetBase = TRUE;
//                        }
//                    }
//                    else
//                    {
//                        // They lifted the pen, see if we will accept coordinate.
//                        if ( CalibrationSampleCount >= MIN_CAL_COUNT )
//                        {
//                            fGotSample = TRUE;
//                        }
//                        else
//                        {
//                            CalibrationState = CalibrationWaiting;
//                        }
//                    }
//        
//                    if ( fGotSample )
//                    {
//                        CalibrationState = CalibrationValid;
//                        lCalibrationXCoord = CX;
//                        lCalibrationYCoord = CY;
//                        SetEvent(hCalibrationSampleAvailable);
//                    }
//                }
//            }
//            else
            {

//kprintf("Raw %x", TouchInputEvent.u.Mouse.State);
//kprintf(" %d", RawX);
//kprintf(" %d\r\n", RawY);

                if( fSample & TOUCH_SAMPLE_CALIBRATED )
                {   // Sample already calibrated
                    CalX = RawX;
                    CalY = RawY;
                }
                else
                {   // Not previously calibrated, do it now.
                    //TouchPanelCalibrateAPoint( RawX, RawY, &CalX, &CalY );
                    TouchCalibrate( RawX, RawY, &CalX, &CalY );
                    fSample |= TOUCH_SAMPLE_CALIBRATED;
                }
        
                // Bounds check this value
                if( CalX < 0 )
                    CalX = 0;
                else
                if( TSP_LCDX && ((UINT32)CalX >= TSP_LCDX) )
                    CalX = TSP_LCDX - X_SCALE_FACTOR;
        
                if( CalY < 0 )
                    CalY = 0;
                else
                if( TSP_LCDY && ((UINT32)CalY >= TSP_LCDY) )
                    CalY = TSP_LCDY - Y_SCALE_FACTOR ;
        
        
                TouchInputEvent.Type = INPUTEVENT_TYPE_MOUSE;

#if 0        
//kprintf("fSample = %x", fSample);
                if (fSample & TOUCH_SAMPLE_PREVIOUSDOWN)
                {
                    if (fSample & TOUCH_SAMPLE_DOWN)
                        TouchInputEvent.u.Mouse.State = MOUSE_STATE_MOVE;
                    else
                    {
                        TouchInputEvent.u.Mouse.State = MOUSE_STATE_LEFTUP;
        
                        //gpINTregs->rINTMSK &= ~BIT_EINT9;
                    }
                }
                else
                {
                    if (fSample & TOUCH_SAMPLE_DOWN)
                        TouchInputEvent.u.Mouse.State = MOUSE_STATE_LEFTDOWN;
                    else
                        continue;
                }
#else

#if 1
                TouchInputEvent.u.Mouse.State = 0;
                if (fSample & TOUCH_SAMPLE_DOWN)
                    TouchInputEvent.u.Mouse.State |= MOUSE_STATE_LEFTDOWN;

                if (fSample & TOUCH_SAMPLE_PREVIOUSDOWN)
                    TouchInputEvent.u.Mouse.State |= MOUSE_STATE_MOVE;

				if( TouchInputEvent.u.Mouse.State == 0 )
					continue;

#else
                if (fSample & TOUCH_SAMPLE_PREVIOUSDOWN)
                {
                    if (fSample & TOUCH_SAMPLE_DOWN)
                        TouchInputEvent.u.Mouse.State = MOUSE_STATE_MOVE;
                    else
                    {
                        TouchInputEvent.u.Mouse.State = MOUSE_STATE_LEFTUP;
        
                        //gpINTregs->rINTMSK &= ~BIT_EINT9;
                    }
                }
                else
                {
                    if (fSample & TOUCH_SAMPLE_DOWN)
                        TouchInputEvent.u.Mouse.State = MOUSE_STATE_LEFTDOWN;
                    else
                        continue;
                }
#endif

#endif        

        
                if( (Smouse != TouchInputEvent.u.Mouse.State) || 
                    (Xmouse != CalX) || 
                    (Ymouse != CalY) )
                {
//kprintf("CAL %x", TouchInputEvent.u.Mouse.State);
//kprintf(" %d", CalX);
//kprintf(" %d\r\n", CalY);
                    TouchInputEvent.u.Mouse.X = CalX;
                    TouchInputEvent.u.Mouse.Y = CalY;
                    SendInputEvent( &TouchInputEvent);

                    
                    Smouse = TouchInputEvent.u.Mouse.State;
                    Xmouse = CalX;
                    Ymouse = CalY;
                }
            }

            continue;
        }
        case MESSAGE_TYPE_SIGNAL:
            continue;

        case MESSAGE_TYPE_DEVICE_CONTROL:
        {
            Message.Type = FALSE;
            break;
        }

        default:
            Message.Type = FALSE;
            break;
        }

        MessageReply( &Message );
    }

    return TRUE;    
}





